//
//  CCIsometric.h
//  CCIsometric
//
//  Created by Shotaro Emori on 12/02/18.
//  Copyright Shotaro Emori 2012. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "cocos2d.h"
#import "CCIsometricTile.h"
#import "CCIsometricLayer.h"
#import "CCIsometricEntity.h"

@class CCIsometricTile, CCIsometricEntity;

@interface CCIsometric : CCNode
{
@public
    CGSize size_;
    ccQuad2 quad_;
    float tileSize_;
    ccIsoSize mapSize_;
    CGSize stageSize_;
    float degrees_;
    CCArray *tiles_;
    float maxScale_;
    float minScale_;
    bool isDrawLine_;
    CCIsometricEntity *selectEntity_;
    CCIsometricEntity *lastSelectEntity_;
    NSMutableArray *areas_;
    ccIsoSize areaSize_;
    NSMutableArray *disableAreaNumbers_;
@private
    CCIsometricTile *tempTile_;
}

@property (assign, readonly) CGSize size;
@property (assign, readonly) ccQuad2 quad;

/*
 * the tiles's size property measured in points 
 */
@property (assign, readonly) float tileSize;
/*
 * the map's size property measured in tiles 
 */
@property (assign, readonly) ccIsoSize mapSize;

/*
 * the stage size property measured in points 
 */
@property (assign, readonly) CGSize stageSize;

/*
 * degrees of a tile
 */
@property (assign, readonly) float degrees;
/*
 * tiles
 */
@property (assign, readonly) CCArray *tiles;

/*
 * max scale
 */
@property (assign, readwrite) float maxScale;

/*
 * min scale
 */
@property (assign, readwrite) float minScale;

@property (assign, readwrite) bool isDrawLine;
@property (assign, readwrite) CCIsometricEntity *selectEntity;
@property (assign, readwrite) CCIsometricEntity *lastSelectEntity;
@property (assign, readonly) NSMutableArray *areas;
@property (assign, readonly) ccIsoSize areaSize;
@property (assign, readonly) NSMutableArray *disableAreaNumbers;

#pragma mark - static method

+ (CCIsometric *)isoWithTileSize:(float)tileSize mapSize:(ccIsoSize)mapSize stageSize:(CGSize)stageSize degrees:(float)degrees;
+ (BOOL)quad:(ccQuad2)quad containsPoint:(CGPoint)point;

#pragma mark - public method

- (id)initWithTileSize:(float)tileSize mapSize:(ccIsoSize)mapSize stageSize:(CGSize)stageSize degrees:(float)degrees;

- (void)divisionArea:(ccIsoSize)size;
- (void)enableAreas:(NSArray *)areas;
- (void)enableArea:(int)areaNumber;
- (void)disableArea:(int)areaNumber;

- (CCIsometricTile *)tileAtIsoPoint:(ccIsoPoint)point;
- (CCIsometricTile *)tileAtPoint:(CGPoint)point;

- (CCIsometricEntity *)entityAtIsoPoint:(ccIsoPoint)point;
- (CCIsometricEntity *)entityAtPoint:(CGPoint)point;

- (BOOL)selectTileWithRect:(CGRect)rect;
- (BOOL)containsTilesWithEntity:(CCIsometricEntity *)entity;
- (BOOL)containsTilesWithPosition:(ccIsoPoint)position size:(ccIsoSize)size;
@end
