//
//  CCIsometric.m
//  CCIsometric
//
//  Created by Shotaro Emori on 12/02/18.
//  Copyright Shotaro Emori 2012. All rights reserved.
//

#import "CCIsometric.h"
#import "CCIsometricTile.h"
#import "CCIsometricEntity.h"

#pragma mark - private method

@interface CCIsometric (Private)
- (void)createMap;
@end

@implementation CCIsometric

@synthesize size = size_,
            quad = quad_,
            tileSize = tileSize_,
            mapSize = mapSize_,
            stageSize = stageSize_,
            degrees = degrees_,
            tiles = tiles_,
            maxScale = maxScale_,
            minScale = minScale_,
            isDrawLine = isDrawLine_,
            selectEntity = selectEntity_,
            lastSelectEntity = lastSelectEntity_,
            areas = areas_,
            areaSize = areaSize,
            disableAreaNumbers = disableAreaNumbers_;

#pragma mark - static method

+ (id)node
{
    NSAssert(0, @"Not support method.");
    return nil;
}

+ (CCIsometric *)isoWithTileSize:(float)tileSize mapSize:(ccIsoSize)mapSize  stageSize:(CGSize)stageSize degrees:(float)degrees
{
    CCIsometric *iso = [[[CCIsometric alloc] initWithTileSize:tileSize mapSize:mapSize stageSize:stageSize degrees:degrees] autorelease];
    return iso;
}

+ (BOOL)quad:(ccQuad2)quad containsPoint:(CGPoint)point {
    if (ccpCross(CGPointMake((quad.tr.x - quad.tl.x), (quad.tr.y - quad.tl.y)), CGPointMake((point.x - quad.tl.x), (point.y - quad.tl.y))) <= 0 &&
        ccpCross(CGPointMake((quad.br.x - quad.tr.x), (quad.br.y - quad.tr.y)), CGPointMake((point.x - quad.tr.x), (point.y - quad.tr.y))) <= 0 &&
        ccpCross(CGPointMake((quad.bl.x - quad.br.x), (quad.bl.y - quad.br.y)), CGPointMake((point.x - quad.br.x), (point.y - quad.br.y))) <= 0 &&
        ccpCross(CGPointMake((quad.tl.x - quad.bl.x), (quad.tl.y - quad.bl.y)), CGPointMake((point.x - quad.bl.x), (point.y - quad.bl.y))) <= 0) {
        return (YES);
    }
    return (NO);
}

#pragma mark - cycle

- (id)init
{
    NSAssert(0, @"Not support method.");
    return nil;
}

- (id)initWithTileSize:(float)tileSize mapSize:(ccIsoSize)mapSize stageSize:(CGSize)stageSize degrees:(float)degrees
{
    self = [super init];
    if (self) {
        tileSize_ = tileSize;
        mapSize_ = mapSize;
        stageSize_ = stageSize;
        degrees_ = degrees;
        tiles_ = [[CCArray alloc] init];
        minScale_ = 0.5;
        maxScale_ = 1.0;
        isDrawLine_ = NO;
        areas_ = [[NSMutableArray alloc] init];
        disableAreaNumbers_ = [[NSMutableArray alloc] init];
        [self createMap];
        
        size_ = CGSizeMake(tileSize * mapSize.width, tileSize * mapSize.height);
        
        CCIsometricTile *tile = ((CCIsometricTile *)[[tiles_ objectAtIndex:0] objectAtIndex:0]);
        self.contentSize = CGSizeMake(tile.contentSize.width * (mapSize_.width + mapSize_.height),
                                      tile.contentSize.height * (mapSize_.width + mapSize_.height));
        
        float tw = tile.contentSize.width/2;
        float th = tile.contentSize.height/2;
        quad_ = (ccQuad2) {
            (ccVertex2F) {-tw * mapSize_.height,                       th * mapSize_.height},
            (ccVertex2F) {tw * mapSize_.width -tw * mapSize_.height,   th * (mapSize_.width + mapSize_.height)},
            (ccVertex2F) {0,                                           0},
            (ccVertex2F) {tw * mapSize_.width,                         th * mapSize_.width},
        };
        
        tempTile_ = [[tiles_ objectAtIndex:0] objectAtIndex:0];
    }
    return (self);
}

- (void)dealloc {
    [areas_ removeAllObjects];
    [areas_ release], areas_ = nil;
    [tiles_ removeAllObjects];
    [tiles_ release], tiles_ = nil;
    [self removeAllChildrenWithCleanup:YES];
    [super dealloc];
}

#pragma mark - public method

- (void)divisionArea:(ccIsoSize)size
{
    NSAssert(self.mapSize.width % size.width == 0,@"error");
    NSAssert(self.mapSize.height % size.height == 0,@"error");
    
    [areas_ removeAllObjects];
    [areas_ release], areas_ = nil;
    areas_ = [[NSMutableArray alloc] init];
    [disableAreaNumbers_ removeAllObjects];
    [disableAreaNumbers_ release], disableAreaNumbers_ = nil;
    disableAreaNumbers_ = [[NSMutableArray alloc] init];
    areaSize_ = size;
    
    for (int i = 0; i < self.mapSize.height / size.height; i++) {
        for (int j = 0; j < self.mapSize.width / size.width; j++) {
            NSMutableArray *a = [[NSMutableArray alloc] init];
            int startY = size.height * i;
            int startX = size.width * j;
            for (int k = 0; k < size.height; k++) {
                for (int l = 0; l < size.width; l++) {
                    [a addObject:[[tiles_ objectAtIndex:startY + k] objectAtIndex:startX + l]];
                }
            }
            [areas_ addObject:a];
            [a release];
        }
    }
}

- (void)enableAreas:(NSArray *)areas
{
    for (int i = 0; i < areas_.count; i++) {
        if ([areas containsObject:I2N(i)]) {
            [self enableArea:i];
        } else {
            [self disableArea:i];
        }
    }
}

- (void)enableArea:(int)areaNumber
{
    NSAssert(areaNumber < areas_.count, @"error");
    
    NSInteger index = [disableAreaNumbers_ indexOfObject:I2N(areaNumber)];
    if (index != NSNotFound) {
        [disableAreaNumbers_ removeObjectAtIndex:index];
    }
    for (int i=0; i<[[areas_ objectAtIndex:areaNumber] count]; i++) {
        ((CCIsometricTile *)[[areas_ objectAtIndex:areaNumber] objectAtIndex:i]).enable = YES;
    }
}

- (void)disableArea:(int)areaNumber
{
    NSAssert(areaNumber < areas_.count, @"error");
    
    NSInteger index = [disableAreaNumbers_ indexOfObject:I2N(areaNumber)];
    if (index == NSNotFound) {
        [disableAreaNumbers_ addObject:I2N(areaNumber)];
    }
    for (int i=0; i<[[areas_ objectAtIndex:areaNumber] count]; i++) {
        ((CCIsometricTile *)[[areas_ objectAtIndex:areaNumber] objectAtIndex:i]).enable = NO;
    }
}

- (CCIsometricTile *)tileAtIsoPoint:(ccIsoPoint)point
{
    if (point.x >= mapSize_.width | point.y >= mapSize_.height) return nil;
    return [[tiles_ objectAtIndex:point.y] objectAtIndex:point.x];
}

- (CCIsometricTile *)tileAtPoint:(CGPoint)point
{
    if (![CCIsometric quad:quad_ containsPoint:point]) {
        return nil;
    }
    CGPoint intersectPoint = ccpIntersectPoint(
                                      [[[tiles_ objectAtIndex:0] objectAtIndex:0] position],
                                      CGPointMake(self.contentSize.width, self.contentSize.width * tan(CC_DEGREES_TO_RADIANS(degrees_))),
                                      CGPointMake(point.y * tan(CC_DEGREES_TO_RADIANS(90-degrees_)) + point.x, 0),
                                      point
                                      );
    float x_ = sqrt(pow(intersectPoint.x, 2) + pow(intersectPoint.y, 2));
    float y_ = sqrt(pow(abs(intersectPoint.x - point.x), 2) + pow(abs(intersectPoint.y - point.y), 2));
    int w = floor(x_/tileSize_);
    int h = floor(y_/tileSize_);
    
    if (h >= mapSize_.height || w >= mapSize_.width) {
        return nil;
    } 
    return (CCIsometricTile *)[[tiles_ objectAtIndex:h] objectAtIndex:w];
}

- (CCIsometricEntity *)entityAtIsoPoint:(ccIsoPoint)point
{
    CCIsometricTile *tile = [self tileAtIsoPoint:point];
    return [tile onEntity];
}

- (CCIsometricEntity *)entityAtPoint:(CGPoint)point
{
    CCIsometricTile *tile = [self tileAtPoint:point];
    return [tile onEntity];
}

- (BOOL)selectTileWithRect:(CGRect)tileRect
{
    CCIsometricTile *tile = [self tileAtPoint:tileRect.origin];
    if (tile == nil) {
        return NO;
    }
    int w = floor(tileRect.size.width / tileSize_);
    int h = floor(tileRect.size.height / tileSize_);
    bool over = YES;
    for (int i = tile.tilePosition.y; i <= tile.tilePosition.y + h; i++) {
        for (int j = tile.tilePosition.x; j <= tile.tilePosition.x + w; j++) {
            if (i >= mapSize_.height || j >= mapSize_.width) {
                over = NO;
            } else {
                ((CCIsometricTile *)[[tiles_ objectAtIndex:i] objectAtIndex:j]).isSelect = YES;
            }
        }
    }
    return over;
}
- (BOOL)containsTilesWithEntity:(CCIsometricEntity *)entity
{
    if (!entity.isoMode) {
        return NO;
    }
    for (int i = entity.tilePosition.y; i < entity.tilePosition.y + entity.tileSize.height; i++) {
        for (int j = entity.tilePosition.x; j < entity.tilePosition.x + entity.tileSize.width; j++) {
            if (i >= mapSize_.height || j >= mapSize_.width) {
                return NO;
            }
            [[tiles_ objectAtIndex:i] objectAtIndex:j];
        }
    }
    return YES;
}

- (BOOL)containsTilesWithPosition:(ccIsoPoint)position size:(ccIsoSize)size
{
    CCIsometricTile *tile_ = nil;
    for (int i = position.y; i < position.y + size.height; i++) {
        for (int j = position.x; j < position.x + size.width; j++) {
            if (i >= mapSize_.height || j >= mapSize_.width) {
                return NO;
            }
            tile_ = [[tiles_ objectAtIndex:i] objectAtIndex:j];
        }
    }
    if (tile_ == nil) {
        return NO;
    }
    return YES;
}

- (void)draw {
    [super draw];
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    if (isDrawLine_) {
        ccDrawColor4F(1.0, 1.0, 1.0, 0.2);
        glLineWidth(1.0f);
        for (int i = 0; i <= mapSize_.height; i++) {
            ccDrawLine(CGPointMake(quad_.bl.x - tempTile_.contentSize.width/2*i, 
                                   quad_.bl.y + tempTile_.contentSize.height/2*i), 
                       CGPointMake(quad_.br.x - tempTile_.contentSize.width/2*i, 
                                   quad_.br.y + tempTile_.contentSize.height/2*i));
        }
        for (int i = 0; i <= mapSize_.width; i++) {
            ccDrawLine(CGPointMake(quad_.bl.x + tempTile_.contentSize.width/2*i, 
                                   quad_.bl.y + tempTile_.contentSize.height/2*i), 
                       CGPointMake(quad_.tl.x + tempTile_.contentSize.width/2*i, 
                                   quad_.tl.y + tempTile_.contentSize.height/2*i));
        }
    }
    for (int i = 0; i < disableAreaNumbers_.count; i++) {
        int j = [[disableAreaNumbers_ objectAtIndex:i] intValue];
        CCIsometricTile *tile = [[areas_ objectAtIndex:j] objectAtIndex:0];
        float tw = tile.contentSize.width / 2;
        float th = tile.contentSize.height / 2;
        float offsetX = tile.position.x;
        float offsetY = tile.position.y;
        CGPoint points[]  = {
            CGPointMake(offsetX, offsetY),
            CGPointMake(-tw * areaSize_.height + offsetX, th * areaSize_.height + offsetY),
            CGPointMake(tw * areaSize_.width - tw * areaSize_.height + offsetX, th * (areaSize_.width + areaSize_.height) + offsetY),
            CGPointMake(tw * areaSize_.width + offsetX, th * areaSize_.width + offsetY),
        };
        ccDrawSolidPoly(points, 4, (ccColor4F){0.0, 0.0, 0.0, 0.2});
    }
}

#pragma mark - private method

- (void)createMap
{
    if (tiles_) {
        [tiles_ removeAllObjects];
        [tiles_ release], tiles_ = nil;
        [self removeAllChildrenWithCleanup:YES];
    }
    tiles_ = [[CCArray alloc] init];
    for (int i = 0; i < mapSize_.height; i++) {
        [tiles_ addObject:[CCArray array]];
        for (int j = 0; j < mapSize_.width; j++) {
            CCIsometricTile *tile = [[CCIsometricTile alloc] initWithSize:tileSize_ degrees:degrees_];
            float x = j * tile.contentSize.width/2  - i * tile.contentSize.width/2;
            float y = j * tile.contentSize.height/2 + i * tile.contentSize.height/2;
            tile.position = CGPointMake(x, y);
            tile.tag = (j<<16) | i;
            tile.tilePosition = (ccIsoPoint) { j, i };
            [self addChild:tile z:0 tag:mapSize_.width * j + i];
            [[tiles_ objectAtIndex:i] addObject:tile];
            [tile release];
        }
    }
}

@end
