//
//  CCIsometricEntity.m
//  CCIsometric
//
//  Created by Shotaro Emori on 12/02/18.
//  Copyright Shotaro Emori 2012. All rights reserved.
//

#import "CCIsometricEntity.h"
#import "CCIsometricLayer.h"
#import "CCIsometric.h"

@implementation CCIsometricEntity

@synthesize touchable    = touchable_,
            tilePosition = tilePosition_,
            tileSize = tileSize_,
            draggable = draggable_,
            isoMode = isoMode_,
            lastPutTilePosition = lastPutTilePosition_;

#pragma mark - cycle

- (id)init
{
    self = [super init];
    if (self) {
        self.draggable = YES;
        self.isoMode = YES;
        touchArray_ = [[NSMutableArray alloc] init];
    }
    return (self);
}

- (void)dealloc
{
    if (((CCIsometricLayer *)self.parent).iso.selectEntity == self) {
        ((CCIsometricLayer *)self.parent).iso.selectEntity = nil;
    }
    [super dealloc];
}

#pragma mark - accsesa method

- (void)setParent:(CCNode *) parent
{
    if (parent != nil) {
        NSAssert([parent isKindOfClass:[CCIsometricLayer class]], @"It can't be added to not CCIsometricLayer");
    }
    [super setParent:parent];
}

- (ccQuad2)quad
{
    CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;
    float tileSize = iso.tileSize;
    int degrees = iso.degrees;
    return (ccQuad2) {
            (ccVertex2F) {
                -tileSize * self.tileSize.height * cos(CC_DEGREES_TO_RADIANS(degrees)) + tileSize * self.tileSize.height,
                tileSize * self.tileSize.height * sin(CC_DEGREES_TO_RADIANS(degrees))
            },
            (ccVertex2F) {
                tileSize * self.tileSize.width * cos(CC_DEGREES_TO_RADIANS(degrees)) - tileSize * self.tileSize.height * cos(CC_DEGREES_TO_RADIANS(degrees)) + tileSize * self.tileSize.height,
                tileSize * (self.tileSize.height + self.tileSize.width) * sin(CC_DEGREES_TO_RADIANS(degrees))
            },
            (ccVertex2F) {
                0 + tileSize * self.tileSize.height,
                0
            },
            (ccVertex2F) {
                tileSize * self.tileSize.width * cos(CC_DEGREES_TO_RADIANS(degrees)) + tileSize * self.tileSize.height,
                tileSize * self.tileSize.width * sin(CC_DEGREES_TO_RADIANS(degrees))
            },
    };
}

- (ccIsoPoint)tilePosition
{
    return tilePosition_;
}

- (void)setTilePosition:(ccIsoPoint)tilePosition
{
    NSAssert(self.parent, @"Parent must be non-nil");
    
    CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;
    CCIsometricTile *tile = [iso tileAtIsoPoint:tilePosition];
    
    if (self.isoMode == NO) {
        self.position = ccp(tile.layerSpacePosition.x - (self.contentSize.width / 2 * self.scale), tile.layerSpacePosition.y);
        tilePosition_ = tilePosition;
    } else if (self.isoMode) {
        for (int i = self.tilePosition.y; i < self.tilePosition.y + self.tileSize.height; i++) {
            if (i < 0 || i >= iso.mapSize.height) {
                continue;
            }
            for (int j = self.tilePosition.x; j < self.tilePosition.x + self.tileSize.width; j++) {
                if (j < 0 || j >= iso.mapSize.width) {
                    continue;
                }
                if (((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity == self) {
                    ((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity = nil;
                }
            }
        }
        
        self.position = ccp(tile.layerSpacePosition.x - (self.tileSize.height * tile.size), tile.layerSpacePosition.y);
        tilePosition_ = tilePosition;
        
        if (self.canPutEntity == NO) {
            CCLOG(@"NO!!!");
        } else {
            for (int i = self.tilePosition.y; i < self.tilePosition.y + self.tileSize.height; i++) {
                if (i < 0 || i >= iso.mapSize.height) {
                    continue;
                }
                for (int j = self.tilePosition.x; j < self.tilePosition.x + self.tileSize.width; j++) {
                    if (j < 0 || j >= iso.mapSize.width) {
                        continue;
                    }
                    ((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity = self;
                }
            }
        }
    }
}

- (BOOL)canPutEntity
{
    CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;

    BOOL canPutEntity_ = YES;
    for (int i = self.tilePosition.y; i < self.tilePosition.y + self.tileSize.height; i++) {
        if (i < 0 || i >= iso.mapSize.height) {
            canPutEntity_ = NO;
            continue;
        }
        for (int j = self.tilePosition.x; j < self.tilePosition.x + self.tileSize.width; j++) {
            if (j < 0 || j >= iso.mapSize.width) {
                canPutEntity_ = NO;
                continue;
            }
            if (((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity != nil &&
                ((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity != self) {
                canPutEntity_ = NO;
                break;
            }
            if (((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).enable == NO) {
                canPutEntity_ = NO;
                break;
            }
        }
    }
    return canPutEntity_;
}

#pragma mark - CCNode method

- (void)draw {
    [super draw];
}

- (void)onEnter
{
    [super onEnter];
    [[[CCDirector sharedDirector] touchDispatcher] addStandardDelegate:self priority:1-self.zOrder];
}

- (void)onExit
{
    if (self.isoMode) {
        CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;
        for (int i = self.tilePosition.y; i < self.tilePosition.y + self.tileSize.height; i++) {
            for (int j = self.tilePosition.x; j < self.tilePosition.x + self.tileSize.width; j++) {
                if(((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity == self) {
                    ((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity = nil;
                };
            }
        }
    }
    [[[CCDirector sharedDirector] touchDispatcher] removeDelegate:self];
    
    if (((CCIsometricLayer *)self.parent).iso.selectEntity == self) {
        ((CCIsometricLayer *)self.parent).iso.selectEntity = nil;
    }
    
    [super onExit];
}

#pragma mark - public method

- (BOOL)containsTouchLocation:(UITouch *)touch
{
    if (self.contentSize.width == 0 && self.contentSize.height == 0) {
        return NO;
    }
    return CGRectContainsPoint(CGRectMake(0, 0, self.contentSize.width, self.contentSize.height), [self convertTouchToNodeSpaceAR:touch]);
}

- (void)creanTileOnEntity
{
    CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;
    if (self.isoMode) {
        for (int i = self.tilePosition.y; i < self.tilePosition.y + self.tileSize.height; i++) {
            if (i < 0 || i >= iso.mapSize.height) {
                continue;
            }
            for (int j = self.tilePosition.x; j < self.tilePosition.x + self.tileSize.width; j++) {
                if (j < 0 || j >= iso.mapSize.width) {
                    continue;
                }
                if (((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity == self) {
                    ((CCIsometricTile *)[[iso.tiles objectAtIndex:i] objectAtIndex:j]).onEntity = nil;
                }
            }
        }
    } else {
        NSAssert(0, @"error");
    }
}

#pragma mark - CCTargetedTouchDelegate

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    for (UITouch *touch in [touches allObjects]) {
        [touchArray_ addObject:touch];
    }
    if (touchArray_.count == 1) {
        movePoint_ = [self.parent convertTouchToNodeSpaceAR:[touchArray_ objectAtIndex:0]];
        isTouch_   = [self containsTouchLocation:[touchArray_ objectAtIndex:0]] && 
                    ((CCIsometricLayer *)self.parent).iso.selectEntity == nil && 
                    self.touchable;
        if (isTouch_ && self.draggable) {
            if (self.canPutEntity) {
                lastPutTilePosition_ = tilePosition_;
            }
            [self.parent reorderChild:self z:1000];
        }
        if (isTouch_) {
            ((CCIsometricLayer *)self.parent).iso.selectEntity = self;
            ((CCIsometricLayer *)self.parent).iso.lastSelectEntity = self;
        }
    }
}

- (void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (touchArray_.count == 1) {
        CGPoint _movePoint_ = [self.parent convertTouchToNodeSpaceAR:[touchArray_ objectAtIndex:0]];
        if (isTouch_) {
            if (self.draggable) {
                CGPoint diff;
                diff.x = movePoint_.x - _movePoint_.x;
                diff.y = movePoint_.y - _movePoint_.y;
                if (self.isoMode) {
                    CCIsometric *iso = ((CCIsometricLayer *)self.parent).iso;
                    CCIsometricTile *tile = [iso tileAtPoint:ccp(_movePoint_.x- iso.position.x, _movePoint_.y - iso.position.y)];
                    if (tile != nil && [iso containsTilesWithPosition:tile.tilePosition size:self.tileSize]) {
                        [self setTilePosition:tile.tilePosition];
                    }
                } else {
                    [self setPosition:ccp(self.position.x - diff.x, self.position.y - diff.y)];
                }
            }
        }
        movePoint_ = _movePoint_;
    }
}

- (void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (isTouch_) {
        if (self.draggable) {
            [self.parent reorderChild:self z:-self.position.y];
        }
        isTouch_ = NO;
        ((CCIsometricLayer *)self.parent).iso.selectEntity = nil;
    }
    [touchArray_ release], touchArray_ = nil;
    touchArray_ = [[NSMutableArray alloc] init];
}

- (void)ccTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (isTouch_) {
        if (self.draggable) {
            [self.parent reorderChild:self z:-self.position.y];
        } 
        isTouch_ = NO;
        ((CCIsometricLayer *)self.parent).iso.selectEntity = nil;
    }
    [touchArray_ release], touchArray_ = nil;
    touchArray_ = [[NSMutableArray alloc] init];
}

@end
