//
//  CCIsometricLayer.m
//  CCIsometric
//
//  Created by Shotaro Emori on 12/02/18.
//  Copyright Shotaro Emori 2012. All rights reserved.
//

#import "CCIsometricLayer.h"
#import "CCIsometric.h"
#import "CCIsometricTile.h"
#import "CCIsometricEntity.h"

@implementation CCIsometricLayer

@synthesize iso = iso_;

#pragma mark - static method

+ (id)node
{
    NSAssert(0, @"Not support method.");
    return nil;
}

#pragma mark - cycle

- (id)init
{
    NSAssert(0, @"Not support method.");
    return nil;
}

- (id)initWithIso:(CCIsometric *)iso
{
    self = [super init];
    if (self) {
        self.anchorPoint = ccp(0, 0);
        self.isTouchEnabled = YES;
        self.contentSize = CGSizeMake(iso.stageSize.width,iso.stageSize.height);
        touchArray = [[NSMutableArray alloc] init];
        iso_ = [iso retain];
        [self addChild:iso_ z:-999];
    }
    return (self);
}

- (void)dealloc
{
    [touchArray release], touchArray = nil;
    [iso_ release], iso_ = nil;
    [super dealloc];
}

#pragma mark - public method

- (void)centerToPoint:(CGPoint)point scale:(float)scale callback:(CCCallFunc *)callback
{
    [self centerToPoint:point duration:0.5f scale:scale callback:callback];
}

- (void)centerToPoint:(CGPoint)point duration:(float)duration scale:(float)scale callback:(CCCallFunc *)callback
{
    CGSize frameSize = CGSizeMake([[UIScreen mainScreen] bounds].size.height, [[UIScreen mainScreen] bounds].size.width);

    CGPoint toPoint = ccp(-point.x * scale + frameSize.width / 2, -point.y * scale + frameSize.height / 2);
    CCMoveTo *move     = [CCMoveTo actionWithDuration:duration position:toPoint];
    CCScaleTo *scaleTo = [CCScaleTo actionWithDuration:duration scale:scale];
    CCSpawn *spawn = [CCSpawn actions:move, scaleTo, nil];
    CCSequence *sequence = [CCSequence actions:spawn, callback, nil];
    [self runAction:sequence];
}

- (void)centerToEntity:(CCIsometricEntity *)entity scale:(float)scale callback:(CCCallFunc *)callback
{
    CGPoint point = ccp(entity.position.x + entity.contentSize.width * entity.scale / 2, entity.position.y + entity.contentSize.height * entity.scale / 2);
    [self centerToPoint:point scale:scale callback:callback];
}

- (void)centerToEntity:(CCIsometricEntity *)entity duration:(float)duration scale:(float)scale callback:(CCCallFunc *)callback
{
    CGPoint point = ccp(entity.position.x + entity.contentSize.width * entity.scale / 2, entity.position.y + entity.contentSize.height * entity.scale / 2);
    [self centerToPoint:point duration:duration scale:scale callback:callback];
}

#pragma mark - CCStandardTouchDelegate

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event;
{
    for (UITouch *touch in [touches allObjects]) {
        [touchArray addObject:touch];
    }
    CGPoint _movePoint1_ = [[touchArray objectAtIndex:0] locationInView:[CCDirector sharedDirector].view];
    movePoint1_ = [[CCDirector sharedDirector] convertToGL:_movePoint1_];
    if ([touchArray count] == 2) {
        CGPoint _movePoint2_ = [[touchArray objectAtIndex:1] locationInView:[CCDirector sharedDirector].view];
        movePoint2_ = [[CCDirector sharedDirector] convertToGL:_movePoint2_];
    }
}

- (void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    if ([touchArray count] == 2) {
        CGPoint _movePoint1_ = [[touchArray objectAtIndex:0] locationInView:[CCDirector sharedDirector].view];
        _movePoint1_ = [[CCDirector sharedDirector] convertToGL:_movePoint1_];
        CGPoint _movePoint2_ = [[touchArray objectAtIndex:1] locationInView:[CCDirector sharedDirector].view];
        _movePoint2_ = [[CCDirector sharedDirector] convertToGL:_movePoint2_];
        
        float diffX = (movePoint1_.x - movePoint2_.x) - (_movePoint1_.x - _movePoint2_.x);
        float diffY = (movePoint1_.y - movePoint2_.y) - (_movePoint1_.y - _movePoint2_.y);
        
        float diffW = self.scale * self.contentSize.width;
        float diffH = self.scale * self.contentSize.height;
        
        if (fabs(diffX) > fabs(diffY)) {
            self.scale = diffX / 200 + self.scale;
        } else {
            self.scale = diffY / 200 + self.scale;
        }
        if (self.scale < iso_.minScale) {
            self.scale = iso_.minScale;
        } else if (self.scale > iso_.maxScale) {
            self.scale = iso_.maxScale;
        }
        [self setPosition:ccp(self.position.x - (self.scale * self.contentSize.width - diffW) / 2, 
                              self.position.y - (self.scale * self.contentSize.height - diffH) / 2)];
        movePoint1_ = _movePoint1_;
        movePoint2_ = _movePoint2_;
    } else if ([touchArray count] == 1 && (iso_.selectEntity == nil || iso_.selectEntity.draggable == NO)) {
        CGPoint _movePoint1_ = [[touchArray objectAtIndex:0] locationInView:[CCDirector sharedDirector].view];
        _movePoint1_ = [[CCDirector sharedDirector] convertToGL:_movePoint1_];
        CGPoint point = ccp(self.position.x - (movePoint1_.x - _movePoint1_.x), self.position.y - (movePoint1_.y - _movePoint1_.y));
        if (-1 * (self.contentSize.width * self.scale - [[CCDirector sharedDirector] winSize].width) > point.x) {
            point.x       = self.position.x;
            _movePoint1_.x = movePoint1_.x;
        }
        if (point.x > 0) {
            point.x       = self.position.x;
            _movePoint1_.x = movePoint1_.x;
        }
        if (-1 * (self.contentSize.height * self.scale - [[CCDirector sharedDirector] winSize].height) > point.y) {
            point.y       = self.position.y;
            _movePoint1_.y = movePoint1_.y;
        }
        if (point.y > 0) {
            point.y       = self.position.y;
            _movePoint1_.y = movePoint1_.y;
        }
        [self setPosition:point];
        movePoint1_ = _movePoint1_;
    }
}

- (void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if ([touchArray count] == 2) {
        CGPoint movePoint = (CGPoint){
            self.position.x, self.position.y
        };
        if (-1 * (self.contentSize.width * self.scale - [[CCDirector sharedDirector] winSize].width) > self.position.x) {
            movePoint = ccp(-1 * (self.contentSize.width * self.scale - [[CCDirector sharedDirector] winSize].width), movePoint.y);
        }
        if (self.position.x > 0) {
            movePoint = ccp(0, movePoint.y);
        }
        if (-1 * (self.contentSize.height * self.scale - [[CCDirector sharedDirector] winSize].height) > self.position.y) {
            movePoint = ccp(movePoint.x, -1 * (self.contentSize.height * self.scale - [[CCDirector sharedDirector] winSize].height));
        }
        if (self.position.y > 0) {
            movePoint = ccp(movePoint.x, 0);
        }
        if (movePoint.x != self.position.x || movePoint.y != self.position.y) {
            CCMoveTo   *move     = [CCMoveTo actionWithDuration:0.1 position:movePoint];
            CCSequence *sequence = (CCSequence *)[CCSequence actions:move, nil];
            [self runAction:sequence];
        }
    }
    [touchArray release];
    touchArray = [[NSMutableArray alloc] init];
}

- (void)ccTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    if ([touchArray count] == 2) {
        CGPoint movePoint = (CGPoint){
            self.position.x, self.position.y
        };
        if (-1 * (self.contentSize.width * self.scale - [[CCDirector sharedDirector] winSize].width) > self.position.x) {
            movePoint = ccp(-1 * (self.contentSize.width * self.scale - [[CCDirector sharedDirector] winSize].width), movePoint.y);
        }
        if (self.position.x > 0) {
            movePoint = ccp(0, movePoint.y);
        }
        if (-1 * (self.contentSize.height * self.scale - [[CCDirector sharedDirector] winSize].height) > self.position.y) {
            movePoint = ccp(movePoint.x, -1 * (self.contentSize.height * self.scale - [[CCDirector sharedDirector] winSize].height));
        }
        if (self.position.y > 0) {
            movePoint = ccp(movePoint.x, 0);
        }
        if (movePoint.x != self.position.x || movePoint.y != self.position.y) {
            CCMoveTo   *move     = [CCMoveTo actionWithDuration:0.1 position:movePoint];
            CCSequence *sequence = (CCSequence *)[CCSequence actions:move, nil];
            [self runAction:sequence];
        }
    }
    [touchArray release];
    touchArray = [[NSMutableArray alloc] init];
}

#pragma mark - private method

@end
