//
//  CCIsometricTile.m
//  CCIsometric
//
//  Created by Shotaro Emori on 12/02/18.
//  Copyright Shotaro Emori 2012. All rights reserved.
//

#import "CCIsometricTile.h"

@implementation CCIsometricTile

@synthesize enable = enable_,
            size = size_,
            quad = quad_,
            degrees = degrees_,
            tilePosition = tilePosition_,
            layerSpacePosition = layerSpacePosition_,
            isDrawLine = isDrawLine_,
            isSelect = isSelect_,
            lineColor = lineColor_,
            selectColor = selectColor_,
            onEntity = onEntity_;

#pragma mark - static method

+ (CCIsometricTile *)tileWithSize:(float)size degrees:(float)degrees
{
    CCIsometricTile *iso = [[[CCIsometricTile alloc] initWithSize:size degrees:degrees] autorelease];
    return iso;
}

#pragma mark - access method

- (CGPoint)layerSpacePosition
{
    return ccp(self.parent.position.x + self.position.x, self.parent.position.y + self.position.y);
}

#pragma mark - public method

- (id)init
{
    NSAssert(0, @"Not support method");
    return nil;
}

- (id)initWithSize:(float)size degrees:(float)degrees;
{
    self = [super init];
    if (self) {
        enable_ = YES;
        size_ = size;
        degrees_ = degrees;
        quad_ = (ccQuad2) {
            (ccVertex2F) {-size_ * cos(CC_DEGREES_TO_RADIANS(degrees_)), size_ * sin(CC_DEGREES_TO_RADIANS(degrees_))},
            (ccVertex2F) {0,                                             size_ * sin(CC_DEGREES_TO_RADIANS(degrees_)) * 2},
            (ccVertex2F) {0,                                             0},
            (ccVertex2F) {size_ * cos(CC_DEGREES_TO_RADIANS(degrees_)),  size_ * sin(CC_DEGREES_TO_RADIANS(degrees_))},
        };
        self.contentSize = CGSizeMake(quad_.br.x - quad_.tl.x, quad_.tr.y - quad_.bl.y);
        isDrawLine_ = NO;
        isSelect_ = NO;
        lineColor_ = (ccColor4F){0.8, 1.0, 0.76, 0.5};
        selectColor_ = (ccColor4F){0.8, 0.0, 0.0, 0.5};
    }
    return (self);
}

#pragma mark - CCNode method

- (void)draw {    
    [super draw];
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    if (isSelect_) {
        CGPoint points[]  = {
            CGPointMake(quad_.tl.x, quad_.tl.y), 
            CGPointMake(quad_.tr.x, quad_.tr.y), 
            CGPointMake(quad_.br.x, quad_.br.y), 
            CGPointMake(quad_.bl.x, quad_.bl.y)
        };
        ccDrawSolidPoly(points, 4, selectColor_);
    } else if (isDrawLine_) {
        ccDrawColor4F(lineColor_.r, lineColor_.g, lineColor_.b, lineColor_.a);
        glLineWidth(1.0f);
        ccDrawLine(CGPointMake(quad_.tl.x, quad_.tl.y), CGPointMake(quad_.tr.x, quad_.tr.y));
        ccDrawLine(CGPointMake(quad_.tr.x, quad_.tr.y), CGPointMake(quad_.br.x, quad_.br.y));
        ccDrawLine(CGPointMake(quad_.br.x, quad_.br.y), CGPointMake(quad_.bl.x, quad_.bl.y));
        ccDrawLine(CGPointMake(quad_.bl.x, quad_.bl.y), CGPointMake(quad_.tl.x, quad_.tl.y));
    }
}

#pragma mark - CCStandardTouchDelegate

- (void) ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    
}
- (void) ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{

}
- (void) ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    
}
- (void) ccTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{

}
@end
